==> pot-3.6.0.src.tar.gz: pot-3.6/console/console.c <==
/*****************************************************************
 * console.c
 *
 * Copyright(C)2003, Julien Delacroix
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GPL.
 *
 */

#include "../main.h"
#include "console.h"

#define DISPLAY_REINIT printf("\033[0m") 
#define DEFAULTSKIN "default.xml" 

int center(char *chaine, int taille){
  int nb_espace;
  int i,j = 0;
  nb_espace = (taille - strlen(chaine));
  for(i = 0; i < nb_espace / 2; i++)
    printf(" ");
  printf("%s",chaine);
  for(j = i + strlen(chaine); j < taille; j++)
    printf(" ");
  return 0;
}
 
int right(char *chaine, int taille, int nbspaces){
  int nb_espace;
  int i = 0;
  chaine[taille-nbspaces]='\0';
  nb_espace = (taille - strlen(chaine)-nbspaces);
  for(i = 0; i < nb_espace; i++)
    printf(" ");
  printf("%s",chaine);
  for(i = 0; i < nbspaces; i++)
    printf(" ");
  return 0;
}

int left(char *chaine, int taille, int nbspaces){
  int i;
  chaine[taille-nbspaces]='\0';
  for(i = 0; i < nbspaces; i++)
    printf(" ");
  printf("%s",chaine);
  for(i = strlen(chaine); i < taille-nbspaces; i++)
    printf(" ");
  return 0;
}

void print_echap_seq(struct cell *format){
  DISPLAY_REINIT;
  if (format->style != DEFAULT) 
    /* change style */
    printf("\033[%dm", format->style);
  if (format->fg_color != DEFAULT) 
    /* change foreground color */
    printf("\033[%dm",FOREGROUND + format->fg_color);
  if (format->bg_color != DEFAULT)
    /* change background color */
    printf("\033[%dm",BACKGROUND + format->bg_color);
}

void print_text(char *text, int length, int align){
  switch(align){
  case CENTER: center(text, length);
    break;
  case LEFT: left(text, length, 1);
    break;
  case RIGHT: right(text, length, 1);
    break;
  }
}

void display_first_line(struct column *skin){
  struct column *tamp; 
  for (tamp=skin;tamp!=NULL;tamp=tamp->next){
    if (tamp->content!=INVALID){
      print_echap_seq(tamp->title_format);
      print_text(tamp->title,tamp->length, tamp->title_format->alignment);
    }/* end of if */ 
  }/* end of for */
  DISPLAY_REINIT;
  printf("\n");
}

void display_pot(struct column *skin, struct liste_connecte *connecte){
  struct column *tamp; 
  for(tamp=skin;tamp!=NULL;tamp=tamp->next){
    if (tamp->content!=INVALID){
      print_echap_seq(tamp->default_format);
      switch(tamp->content){
      case LOGIN:
	print_text(connecte->login, tamp->length, tamp->default_format->alignment); 
	break;
      case TALK:
	if ( connecte->talk == -2)
	  print_text("w", tamp->length, 
		     tamp->default_format->alignment);
	else if ( connecte->talk == -1)
	  print_text("x", tamp->length, 
		     tamp->default_format->alignment);
	else if ( connecte->talk == 1)
	  print_text("+", tamp->length, 
		     tamp->default_format->alignment);
	else
	  print_text("-", tamp->length, 
		     tamp->default_format->alignment);
	break;
      case IDLE:
	print_text((char*)print_idle(connecte->idle), 
		   tamp->length, tamp->default_format->alignment); 
	break;
      case ROOM:
	print_text(connecte->salle, tamp->length, 
		   tamp->default_format->alignment); 
	break;
      case COMP:
	print_text(connecte->ordi, tamp->length, 
		   tamp->default_format->alignment); 
	break;
      case NAME:
	print_text(connecte->nom, tamp->length, 
		   tamp->default_format->alignment); 
      }/* end of switch */
    }/* end of if */
  }/* end of for */
  DISPLAY_REINIT;
  printf("\n");
}

char *get_skin_file_descriptor(char *filename){
  char *home_dir;
  char *xml_file;
  int home_len;
  struct stat *buf=(struct stat *)malloc(sizeof(struct stat));
  
  /* get home dir */
  home_len=strlen(getenv("HOME"));
  if ((home_dir = (char *)malloc(sizeof(char)*(home_len))) == NULL){
    fprintf( stderr , "Allocation de memoire impossible" );
    exit -1;
  }
  if ((xml_file = (char *)malloc(sizeof(char)*(home_len+30))) == NULL){
    fprintf( stderr , "Allocation de memoire impossible" );
    exit -1;
  }
  strncpy(home_dir, getenv("HOME"), home_len);

  /* Trying .pot/skin in in home dir */
  strncpy(xml_file, home_dir, home_len);
  strcat(xml_file, "/.pot/skin/");
  strncat(xml_file, filename, 15);
  strncat(xml_file, ".xml", 4);
  if(!stat(xml_file, buf))
    return xml_file;
  
  /* Trying skins directory in the source */    
  strncpy(xml_file, SKINDIR, 45);
  strncat(xml_file, filename, 15);
  strncat(xml_file, ".xml", 4);
  if(!stat(xml_file, buf))
    return xml_file;

  /* Trying default skin in .pot/skin in in home dir */
  strncpy(xml_file, home_dir, home_len);
  xml_file[home_len]='\0'; /* sans ca, strcat ajoute pas au bon endroit */
  strcat(xml_file, "/.pot/skin/");
  strcat(xml_file, DEFAULTSKIN);
  if(!stat(xml_file, buf))
    return xml_file;
  
  /* Trying default skin in skins directory in the source */    
  strncpy(xml_file, SKINDIR, 45);
  strcat(xml_file, DEFAULTSKIN);
  if(!stat(xml_file, buf))
    return xml_file;
  
  fprintf(stderr, "Could not find any skin file\n");
  exit(-1);
}


int console(char *chemin, char *skin_file){
  int compteur;
  struct column *skin;
  
  compteur = recherche_pots(chemin);
  
  skin_file=get_skin_file_descriptor(skin_file);
  skin = skin_parse(skin_file);
  
  /* initialisation de la structure */
  connecte = &depart;
  
  if ( compteur > 0 ){
    if(IS_OPTION_NOT_SET(OPTION_QUIET))
      display_first_line(skin);
    
    while ( (connecte = connecte->next) )
      display_pot(skin, connecte);
    return(EXIT_SUCCESS);
  }
  else
    printf ("Pas de \"pot\" trouv.\n");
  return(EXIT_SUCCESS);
}

==> pot-3.7.1.src.tar.gz: pot-3.7/console/console.c <==
/*****************************************************************
 * console.c
 *
 * Copyright(C)2003, Julien Delacroix
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GPL.
 *
 */

#include "../main.h"
#include "console.h"

#define DISPLAY_REINIT if (IS_OPTION_SET(OPTION_COULEUR)) printf("\033[0m") 
#define DEFAULTSKIN "default.xml" 

int center(char *chaine, int taille){
  int nb_espace;
  int i,j = 0;
  nb_espace = (taille - strlen(chaine));
  for(i = 0; i < nb_espace / 2; i++)
    printf(" ");
  printf("%s",chaine);
  for(j = i + strlen(chaine); j < taille; j++)
    printf(" ");
  return 0;
}
 
int right(char *chaine, int taille, int nbspaces){
  int nb_espace;
  int i = 0;
  chaine[taille-nbspaces]='\0';
  nb_espace = (taille - strlen(chaine)-nbspaces);
  for(i = 0; i < nb_espace; i++)
    printf(" ");
  printf("%s",chaine);
  for(i = 0; i < nbspaces; i++)
    printf(" ");
  return 0;
}

int left(char *chaine, int taille, int nbspaces){
  int i;
  chaine[taille-nbspaces]='\0';
  for(i = 0; i < nbspaces; i++)
    printf(" ");
  printf("%s",chaine);
  for(i = strlen(chaine); i < taille-nbspaces; i++)
    printf(" ");
  return 0;
}

void print_echap_seq(struct cell *format){
  if (IS_OPTION_SET(OPTION_COULEUR)) {
    DISPLAY_REINIT;
    if (format->style != DEFAULT) 
      /* change style */
      printf("\033[%dm", format->style);
    if (format->fg_color != DEFAULT) 
      /* change foreground color */
      printf("\033[%dm",FOREGROUND + format->fg_color);
    if (format->bg_color != DEFAULT)
      /* change background color */
      printf("\033[%dm",BACKGROUND + format->bg_color);
  }
}

void print_text(char *text, int length, int align){
  switch(align){
  case CENTER: center(text, length);
    break;
  case LEFT: left(text, length, 1);
    break;
  case RIGHT: right(text, length, 1);
    break;
  }
}

void display_first_line(struct column *skin){
  struct column *tamp; 
  for (tamp=skin;tamp!=NULL;tamp=tamp->next){
    if (tamp->content!=INVALID){
      print_echap_seq(tamp->title_format);
      print_text(tamp->title,tamp->length, tamp->title_format->alignment);
    }/* end of if */ 
  }/* end of for */
  DISPLAY_REINIT;
  printf("\n");
}

void display_pot(struct column *skin, struct liste_connecte *connecte){
  struct column *tamp; 
  for(tamp=skin;tamp!=NULL;tamp=tamp->next){
    if (tamp->content!=INVALID){
      print_echap_seq(tamp->default_format);
      switch(tamp->content){
      case LOGIN:
	print_text(connecte->login, tamp->length, tamp->default_format->alignment); 
	break;
      case TALK:
	if ( connecte->talk == -2)
	  print_text("w", tamp->length, 
		     tamp->default_format->alignment);
	else if ( connecte->talk == -1)
	  print_text("x", tamp->length, 
		     tamp->default_format->alignment);
	else if ( connecte->talk == 1)
	  print_text("+", tamp->length, 
		     tamp->default_format->alignment);
	else
	  print_text("-", tamp->length, 
		     tamp->default_format->alignment);
	break;
      case IDLE:
	print_text((char*)print_idle(connecte->idle), 
		   tamp->length, tamp->default_format->alignment); 
	break;
      case ROOM:
	print_text(connecte->salle, tamp->length, 
		   tamp->default_format->alignment); 
	break;
      case COMP:
	print_text(connecte->ordi, tamp->length, 
		   tamp->default_format->alignment); 
	break;
      case NAME:
	print_text(connecte->nom, tamp->length, 
		   tamp->default_format->alignment); 
      }/* end of switch */
    }/* end of if */
  }/* end of for */
  DISPLAY_REINIT;
  printf("\n");
}

char *get_skin_file_descriptor(char *filename){
  char *home_dir;
  char *xml_file;
  int home_len;
  struct stat *buf=(struct stat *)malloc(sizeof(struct stat));
  
  /* get home dir */
  home_len=strlen(getenv("HOME"));
  if ((home_dir = (char *)malloc(sizeof(char)*(home_len))) == NULL){
    fprintf( stderr , "Allocation de memoire impossible" );
    exit -1;
  }
  if ((xml_file = (char *)malloc(sizeof(char)*(home_len+30))) == NULL){
    fprintf( stderr , "Allocation de memoire impossible" );
    exit -1;
  }
  strncpy(home_dir, getenv("HOME"), home_len);

  /* Trying .pot/skin in in home dir */
  strncpy(xml_file, home_dir, home_len);
  strcat(xml_file, "/.pot/skin/");
  strncat(xml_file, filename, 15);
  strncat(xml_file, ".xml", 4);
  if(!stat(xml_file, buf))
    return xml_file;
  
  /* Trying skins directory in the source */    
  strncpy(xml_file, SKINDIR, 45);
  strncat(xml_file, filename, 15);
  strncat(xml_file, ".xml", 4);
  if(!stat(xml_file, buf))
    return xml_file;

  /* Trying default skin in .pot/skin in in home dir */
  strncpy(xml_file, home_dir, home_len);
  xml_file[home_len]='\0'; /* sans ca, strcat ajoute pas au bon endroit */
  strcat(xml_file, "/.pot/skin/");
  strcat(xml_file, DEFAULTSKIN);
  if(!stat(xml_file, buf))
    return xml_file;
  
  /* Trying default skin in skins directory in the source */    
  strncpy(xml_file, SKINDIR, 45);
  strcat(xml_file, DEFAULTSKIN);
  if(!stat(xml_file, buf))
    return xml_file;
  
  fprintf(stderr, "Could not find any skin file\n");
  exit(-1);
}


int console(char *chemin, char *skin_file){
  int compteur;
  struct column *skin;
  
  compteur = recherche_pots(chemin);
  
  skin_file=get_skin_file_descriptor(skin_file);
  skin = skin_parse(skin_file);
  
  /* initialisation de la structure */
  connecte = &depart;
  
  if ( compteur > 0 ){
    if(IS_OPTION_NOT_SET(OPTION_QUIET))
      display_first_line(skin);
    
    while ( (connecte = connecte->next) )
      display_pot(skin, connecte);
    return(EXIT_SUCCESS);
  }
  else
    printf ("Pas de \"pot\" trouv.\n");
  return(EXIT_SUCCESS);
}
